@@ -834,7 +834,21 @@ class DataFrame:
 
 
 
-        if type(rows) == range:
+        if type(rows) == range or type(rows) == slice:
+            if type(rows) == slice:
+                if rows.start is None:
+                    start = 0
+                else:
+                    start = rows.start
+                if rows.stop is None:
+                    stop = self.__nrow_original__
+                else:
+                    stop = rows.stop
+                if rows.step is None:
+                    step = 1                    
+                else:
+                    step = rows.step                
+                rows = range(start,stop,step)
             return DataFrame.__get_rows_from_to__(self,min(rows),max(rows),has_header = has_header,output_header = output_header,return_other_object = return_other_object)
         else:
 
@@ -950,42 +964,16 @@ class DataFrame:
                 self.__clear_commands__()
                 return __ddf__
 
-    def get_cols(self,cols,has_header = True,output_header = True):
-        awk_command = Awk_command()
-        if type(cols) == np.array:
-            cols = cols.tolist()
 
-        new_cols = np.empty(len(cols),int)
-
-        names = self.names()
-        for i in range(len(cols)):
-            col = cols[i]
-            if type(col) != int:
-                index = np.where(names == col)[0]
-                new_cols[i] = index
-            else:
-                new_cols[i] = col
-        cols = np.unique(new_cols)
-        # if not self.__nrow_modified_to_unknown_value__:
-        cols = cols[np.where(np.isin(cols,np.arange(0,self.__ncol__)))[0]]
+    def __get_cols_from_to__(self,cols,has_header = True,output_header = True):
         awk_command = Awk_command()
-        awk_command.ncol_selected = len(cols)
+        
+        
+        awk_command.ncol_selected = max(cols) - min(cols)
         awk_command.nrow_before = self.__nrow__
         awk_command.ncol_before = self.__ncol__
         self.__ncol__ = awk_command.ncol_selected
 
-        cols_str = np.array2string(cols,separator="\n")
-        cols_str = cols_str[1:len(cols_str)-1]
-        if not os.path.exists(os.path.expanduser('~') + "/.tmp/"):
-            os.mkdir(os.path.expanduser('~') + "/.tmp/")
-        if not os.path.exists(os.path.expanduser('~') + "/.tmp/awk_dataframe/"):
-            os.mkdir(os.path.expanduser('~') + "/.tmp/awk_dataframe/")
-            print("Creating folder ",os.path.expanduser('~') + "/.tmp/awk_dataframe/")
-        path_cols = os.path.expanduser('~') + "/.tmp/awk_dataframe/cols_" + self.__id__ + "_" + awk_command.id + ".txt"
-        # note("TODO: modify this to save with bash")
-        # command = "echo '" + cols_str + "' | sort | xargs -I {} echo \"{}\" >" + path_cols
-        # os.system(command)
-
         variables = "-v has_header=\"has_header_variable\" -v output_header=\"output_header_variable\" -v min_col=\"" + str(min(cols)) + "\" -v max_col=\"" + str(max(cols)) + "\" "
         command = "gawk -v FPAT='([^" + self.__delimiter__ + "]*)|(" + self.__string_delimiter__ + "[^" + self.__string_delimiter__ + "]+" + self.__string_delimiter__ + ")' " + variables + """'
         BEGIN {
@@ -994,12 +982,141 @@ class DataFrame:
 
         }
         {
-            if (FNR==NR){
-                cols[$0+1] = 1
+            if (has_header){
+                if (FNR == 1){
+                    if (output_header){
+                        for (i=min_col+1;i<max_col+1;i++){
+                            
+                            printf("%s""" + self.__delimiter__ + """", $i)
+                            
+                        }
+                        printf("%s", $(max_col+1))
+                        printf("%s" RS_new,"")
+                    }
+                }else{
+                    for (i=min_col+1;i<max_col+1;i++){
+                        
+                        printf("%s""" + self.__delimiter__ + """", $i)
+                        
+                    }
+                    printf("%s", $(max_col+1))
+                    printf("%s" RS_new,"")
+
+                }
             }else{
-                if (has_header){
-                    if (FNR == 1){
-                        if (output_header){
+                for (i=min_col+1;i<max_col+1;i++){
+                    printf("%s""" + self.__delimiter__ + """", $i)
+                }
+                printf("%s", $(max_col+1))
+                printf("%s" RS_new,"")
+            }
+            
+
+        }
+        END {
+
+        }
+        ' """
+        awk_command.command = command
+        if output_header:
+                awk_command.output_header = 1
+        else:
+            awk_command.output_header = 0
+        if has_header:
+                awk_command.has_header = 1
+        else:
+            awk_command.has_header = 0
+        awk_command.priority = 2
+        awk_command.type = "get_cols_from_to"
+        self.__commands__.append(awk_command)
+        id_command = awk_command.id
+        if id_command not in self.__num_copies_commands__.keys():
+            self.__num_copies_commands__[id_command] = 0
+        self.__num_copies_commands__[id_command] += 1
+
+        __ddf__ = self.__deepcopy_internal__({})
+        __ddf__.__settle_commands__()
+        self.__clear_commands__()
+        return __ddf__
+
+    def get_cols(self,cols,has_header = True,output_header = True):
+        if type(cols) == range or type(cols) == slice:
+            if type(cols) == slice:
+                if cols.start is None:
+                    start = 0
+                else:
+                    start = cols.start
+                if cols.stop is None:
+                    stop = self.__ncol_original__
+                else:
+                    stop = cols.stop
+                if cols.step is None:
+                    step = 1                    
+                else:
+                    step = cols.step                
+                cols = range(start,stop,step)
+            return DataFrame.__get_cols_from_to__(self,cols,has_header = has_header,output_header = output_header)
+        else:
+            awk_command = Awk_command()
+            if type(cols) == np.array:
+                cols = cols.tolist()
+
+            new_cols = np.empty(len(cols),int)
+
+            names = self.names()
+            for i in range(len(cols)):
+                col = cols[i]
+                if type(col) != int:
+                    index = np.where(names == col)[0]
+                    new_cols[i] = index
+                else:
+                    new_cols[i] = col
+            cols = np.unique(new_cols)
+            # if not self.__nrow_modified_to_unknown_value__:
+            cols = cols[np.where(np.isin(cols,np.arange(0,self.__ncol__)))[0]]
+            awk_command = Awk_command()
+            awk_command.ncol_selected = len(cols)
+            awk_command.nrow_before = self.__nrow__
+            awk_command.ncol_before = self.__ncol__
+            self.__ncol__ = awk_command.ncol_selected
+
+            cols_str = np.array2string(cols,separator="\n")
+            cols_str = cols_str[1:len(cols_str)-1]
+            if not os.path.exists(os.path.expanduser('~') + "/.tmp/"):
+                os.mkdir(os.path.expanduser('~') + "/.tmp/")
+            if not os.path.exists(os.path.expanduser('~') + "/.tmp/awk_dataframe/"):
+                os.mkdir(os.path.expanduser('~') + "/.tmp/awk_dataframe/")
+                print("Creating folder ",os.path.expanduser('~') + "/.tmp/awk_dataframe/")
+            path_cols = os.path.expanduser('~') + "/.tmp/awk_dataframe/cols_" + self.__id__ + "_" + awk_command.id + ".txt"
+            # note("TODO: modify this to save with bash")
+            # command = "echo '" + cols_str + "' | sort | xargs -I {} echo \"{}\" >" + path_cols
+            # os.system(command)
+
+            variables = "-v has_header=\"has_header_variable\" -v output_header=\"output_header_variable\" -v min_col=\"" + str(min(cols)) + "\" -v max_col=\"" + str(max(cols)) + "\" "
+            command = "gawk -v FPAT='([^" + self.__delimiter__ + "]*)|(" + self.__string_delimiter__ + "[^" + self.__string_delimiter__ + "]+" + self.__string_delimiter__ + ")' " + variables + """'
+            BEGIN {
+                RS = "record_delimiter"
+                RS_new = "record_delimiter_transform"
+
+            }
+            {
+                if (FNR==NR){
+                    cols[$0+1] = 1
+                }else{
+                    if (has_header){
+                        if (FNR == 1){
+                            if (output_header){
+                                for (i=min_col+1;i<max_col+1;i++){
+                                    if (cols[i] == 1){
+                                        printf("%s""" + self.__delimiter__ + """", $i)
+                                    }
+                                }
+                                if (cols[max_col+1] == 1){
+                                    printf("%s", $(max_col+1))
+                                }
+                                printf("%s" RS_new,"")
+                            }
+                        }else{
                             for (i=min_col+1;i<max_col+1;i++){
                                 if (cols[i] == 1){
                                     printf("%s""" + self.__delimiter__ + """", $i)
@@ -1009,6 +1126,7 @@ class DataFrame:
                                 printf("%s", $(max_col+1))
                             }
                             printf("%s" RS_new,"")
+
                         }
                     }else{
                         for (i=min_col+1;i<max_col+1;i++){
@@ -1020,48 +1138,36 @@ class DataFrame:
                             printf("%s", $(max_col+1))
                         }
                         printf("%s" RS_new,"")
-
                     }
-                }else{
-                    for (i=min_col+1;i<max_col+1;i++){
-                        if (cols[i] == 1){
-                            printf("%s""" + self.__delimiter__ + """", $i)
-                        }
-                    }
-                    if (cols[max_col+1] == 1){
-                        printf("%s", $(max_col+1))
-                    }
-                    printf("%s" RS_new,"")
                 }
+
             }
+            END {
 
-        }
-        END {
+            }
+            ' <(echo '""" + cols_str + """') """
+            awk_command.command = command
+            awk_command.path_temp_file = path_cols
+            if output_header:
+                    awk_command.output_header = 1
+            else:
+                awk_command.output_header = 0
+            if has_header:
+                    awk_command.has_header = 1
+            else:
+                awk_command.has_header = 0
+            awk_command.priority = 2
+            awk_command.type = "get_cols"
+            self.__commands__.append(awk_command)
+            id_command = awk_command.id
+            if id_command not in self.__num_copies_commands__.keys():
+                self.__num_copies_commands__[id_command] = 0
+            self.__num_copies_commands__[id_command] += 1
 
-        }
-        ' <(echo '""" + cols_str + """') """
-        awk_command.command = command
-        awk_command.path_temp_file = path_cols
-        if output_header:
-                awk_command.output_header = 1
-        else:
-            awk_command.output_header = 0
-        if has_header:
-                awk_command.has_header = 1
-        else:
-            awk_command.has_header = 0
-        awk_command.priority = 2
-        awk_command.type = "get_cols"
-        self.__commands__.append(awk_command)
-        id_command = awk_command.id
-        if id_command not in self.__num_copies_commands__.keys():
-            self.__num_copies_commands__[id_command] = 0
-        self.__num_copies_commands__[id_command] += 1
-
-        __ddf__ = self.__deepcopy_internal__({})
-        __ddf__.__settle_commands__()
-        self.__clear_commands__()
-        return __ddf__
+            __ddf__ = self.__deepcopy_internal__({})
+            __ddf__.__settle_commands__()
+            self.__clear_commands__()
+            return __ddf__
 
     def __clear_commands__(self):
         new_commands = []
@@ -1398,3 +1504,10 @@ class DataFrame:
                 self.__path__ = path_output
             if __clear_all_commands__:
                 self.__clear_all_commands__()
+
+
+
+def read_csv(path,delimiter = ",",has_header = True,names_columns = [],string_delimiter = "\""):
+    __ddf__ = DataFrame()
+    __ddf__.read_csv(path,delimiter = delimiter,has_header = has_header,names_columns = names_columns,string_delimiter = string_delimiter)
+    return __ddf__
\ No newline at end of file
